# Maxime MORGE <maxime.morge@univ-lyon1.fr>
import os
import asyncio
import json
from typing import Literal, Dict
from pydantic import BaseModel
from autogen_agentchat.agents import AssistantAgent
from autogen_agentchat.messages import TextMessage
from autogen_core import CancellationToken
from autogen_ext.models.openai import OpenAIChatCompletionClient
from enum import Enum

from src.rps.rps_draw_2loop import model

# Load API key from environment variable
OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")
# Validate API key
if not OPENAI_API_KEY:
    raise ValueError("Missing OPENAI_API_KEY. Set it as an environment variable.")

class RestrictedWelfare(Enum):
    EGALITARIAN = "Egalitarian", "You have a stronger preference for fairness between players and maximize the minimum income."
    SELFISH = "Selfish", "You have a stronger preference for your own interest and maximize your own income."
    ALTRUISTIC = "Altruist", "You have a stronger preference for another player’s interest and maximize another player’s income."

    def __init__(self, label, description):
        self._label = label
        self._description = description

    @property
    def label(self):
        return self._label

    @property
    def description(self):
        return self._description


# Define the expected response format as a Pydantic model
class AgentResponse(BaseModel):
    my_share: int
    other_share: int
    reasoning: str


class DictatorContinuousSetup:
    debug = False

    def __init__(self, model: str, temperature: float, preference: RestrictedWelfare, strategy=False):
        self.model = model
        self.temperature = temperature
        self.preference = preference
        self.strategy = strategy

        if not self.strategy:
            is_openai_model = model.startswith("gpt")
            base_url = "https://api.openai.com/v1" if is_openai_model else "http://localhost:11434/v1"

            self.model_info = {
                "temperature": self.temperature,
                "function_calling": True,
                "parallel_tool_calls": True,
                "family": "unknown",
                "json_output": True,
                "vision": False
            }

            self.model_client = OpenAIChatCompletionClient(
                model=self.model,
                base_url=base_url,
                api_key=OPENAI_API_KEY,
                model_info=self.model_info,
                response_format=AgentResponse
            )

    async def run(self) -> Dict:
        """Runs the dictator game and returns a detailed response dictionary."""
        if self.strategy:
            return self.apply_strategy(self.model)

        instruction = f"""
        Hi, let’s play a game. You are paired with another player.
        Your role is to decide how to divide $1000, and the other player simply receives your choice.
        Money can be lost in the process.
        {self.preference.description}
        
        How would you like to divide the money?
    
        Please provide one specific choice, stating how much you keep, how much you give, and how much is lost.
        """
        agent = AssistantAgent(
            name="Dictator",
            model_client=self.model_client,
            system_message="You are a helpful assistant."
        )
        response = await agent.on_messages(
            [TextMessage(content=instruction, source="user")],
            cancellation_token=CancellationToken(),
        )

        response_data = response.chat_message.content
        if self.debug:
            print(f"Raw Response: {response_data}")

        response_dict = json.loads(response_data)
        agent_response = AgentResponse.model_validate(response_dict)

        is_consistent = self.check_consistency(agent_response)
        return {
            "is_consistent": is_consistent,
            "my_share": agent_response.my_share,
            "other_share": agent_response.other_share,
            "reasoning": agent_response.reasoning
        }

    def apply_strategy(self, model: str) -> Dict:
        """Applies a predefined strategy based on the preference."""
        if model == "gpt-4.5-preview-2025-02-27" or model == "llama3" or model == "mistral-small":
            strategy_choices = {
                RestrictedWelfare.SELFISH: {"my_share": 1000, "other_share": 0, "reasoning": "Maximizing personal gain."},
                RestrictedWelfare.ALTRUISTIC: {"my_share": 0, "other_share": 1000, "reasoning": "Maximizing the other player’s gain."},
                RestrictedWelfare.EGALITARIAN: {"my_share": 500, "other_share": 500, "reasoning": "Ensuring a fair split."},
            }
            strategy_response = strategy_choices.get(self.preference, {"my_share": 500, "other_share": 500, "reasoning": "Default fair split."})
            return {
                "is_consistent": True,
                "my_share": strategy_response["my_share"],
                "other_share": strategy_response["other_share"],
                "reasoning": strategy_response["reasoning"]
            }
        if model == "deepseek-R1":
            return  {"error": "Preference strategy not defined"}


    def check_consistency(self, agent_response: AgentResponse) -> bool:
        """Check if the response aligns with the given preference."""
        valid_choices = {
            RestrictedWelfare.SELFISH: (1000, 0),
            RestrictedWelfare.ALTRUISTIC: (0, 1000),
            RestrictedWelfare.EGALITARIAN: (500, 500),
        }

        expected_values = valid_choices.get(self.preference, None)
        if expected_values:
            return (
                    agent_response.my_share == expected_values[0] and
                    agent_response.other_share == expected_values[1]
            )
        return False


# Run the async function and return the response
if __name__ == "__main__":
    preference = RestrictedWelfare.EGALITARIAN
    game_agent = DictatorContinuousSetup(model="llama3", temperature=0.7, preference=preference, strategy=False)
    response = asyncio.run(game_agent.run())
    print(response)
