import asyncio
import os
import pandas as pd
from src.dictator.dictator__continuous_setup import DictatorContinuousSetup, RestrictedWelfare


class DictatorContinuousSetupExperiment:
    debug = True
    def __init__(self, models: list[str], temperature: float, iterations: int, output_file: str):
        self.models = models
        self.temperature = temperature
        self.iterations = iterations
        self.output_file = output_file  # Path to the CSV output file

    # Helper function to escape double quotes in the reasoning string
    def protect_reasoning(self, reasoning):
        if reasoning:
            return f'"{reasoning.replace("\"", "\"\"")}"'
        return reasoning

    async def run_experiment(self):
        preferences = [RestrictedWelfare.SELFISH, RestrictedWelfare.ALTRUISTIC, RestrictedWelfare.EGALITARIAN]
        file_exists = os.path.isfile(self.output_file)  # Check if file already exists
        # Run the dictator game for each model and preference
        for model in self.models:
            if self.debug:
                print(f"Running experiment for model: {model}")
            for preference in preferences:
                print(f"Running with preference: {preference.name}")
                for iteration in range(1, self.iterations + 1):
                    print(f"Iteration: {iteration}")
                    # Initialize the DictatorSetup for the current model and preference
                    game_agent = DictatorContinuousSetup(model=model, temperature=self.temperature, preference=preference)
                    try:
                        agent_response = await game_agent.run()
                        is_consistent = agent_response['is_consistent']
                        my_share = agent_response['my_share']
                        other_share = agent_response['other_share']
                        reasoning = agent_response['reasoning']
                        reasoning = self.protect_reasoning(reasoning)

                    except Exception as e:
                        print(f"Error in iteration {iteration} for model {model} ({preference.name}): {e}")
                        is_consistent = False
                        my_share, other_share, reasoning = None, None, None
                    # Create a single-row DataFrame for the current result
                    df = pd.DataFrame([{
                        'Iteration': iteration,
                        'Model': model,
                        'Temperature': self.temperature,
                        'Preference': preference.name,
                        'is_consistent': is_consistent,
                        'my_share': my_share,
                        'other_share': other_share,
                        'reasoning': reasoning
                    }])
                    # Append results to the CSV file
                    df.to_csv(self.output_file, mode='a', header=not file_exists, index=False)
                    file_exists = True  # Ensure header is only written once
                    if self.debug:
                        print(f"Result for Iteration {iteration} saved to {self.output_file}")

# Running the experiment
if __name__ == "__main__":
    models = ["llama3", "mistral-small", "deepseek-r1"] #"gpt-4.5-preview-2025-02-27"
    temperature = 0.7
    iterations = 30
    output_file = '../../data/dictator/dictator_continuous_setup.csv'

    experiment = DictatorContinuousSetupExperiment(models=models, temperature=temperature, iterations=iterations, output_file=output_file)
    asyncio.run(experiment.run_experiment())

    print(f"Experiment results saved to {output_file}")